import 'package:fis_common/logger/logger.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/define.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:fis_ui/usual/show_dialog.dart';
import 'package:fis_ui/widgets/assist/divider.dart';
import 'package:fis_ui/widgets/container/container.dart';
import 'package:fis_ui/widgets/container/sized_box.dart';
import 'package:fis_ui/widgets/functional/button.dart';
import 'package:fis_ui/widgets/functional/text.dart';
import 'package:fis_ui/widgets/layout/column.dart';
import 'package:fis_ui/widgets/layout/flex.dart';
import 'package:fis_ui/widgets/layout/row.dart';
import 'package:flutter/material.dart';

/// 确认提示框
class FConfirmAlert extends StatelessWidget implements FWidget {
  const FConfirmAlert({
    Key? key,
    required this.title,
    this.subTitle,
    this.confrimLabel,
    this.cancelLabel,
    this.onConfirm,
    this.onCancel,
    this.isCancelPrimary,
    this.isConfirmReadonly,
    this.isShowCancel = true,
    required this.businessParent,
  }) : super(key: key);

  @override
  final FInteractiveContainer businessParent;

  final bool isShowCancel;

  /// 标题
  final String title;

  /// 副标题
  final String? subTitle;

  /// 确认文字
  final String? confrimLabel;

  /// 取消文字
  final String? cancelLabel;

  /// 确认事件
  final VoidCallback? onConfirm;

  /// 取消事件
  final VoidCallback? onCancel;

  /// 确认按钮是否只读
  final bool? isConfirmReadonly;

  ///是否显示主色用于引导用户点击
  final bool? isCancelPrimary;

  @override
  FWidget build(BuildContext context) {
    return FSimpleDialog(
      isNeedClose: false,
      titleColor: Colors.white,
      titleAxisAlignment: MainAxisAlignment.center,
      titlePadding: const EdgeInsets.symmetric(
        vertical: 24,
        horizontal: 26,
      ),
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(8),
      ),
      backgroundColor: Colors.white,
      contentPadding: EdgeInsets.zero,
      isFConfirmAlert: true,
      alignment: Alignment.center,
      title: _buildTitle(),
      onClose: null,
      children: [
        FDivider(
          height: 1,
          color: Colors.grey.shade300,
          indent: 14,
          endIndent: 14,
        ),
        FSizedBox(
          height: 48,
          child: FRow(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              if (isShowCancel) ...[
                FExpanded(child: _buildCancel(context)),
                FVerticalDivider(
                  width: .5,
                  color: Colors.grey.shade300,
                  indent: 10,
                  endIndent: 10,
                ),
              ],
              FExpanded(child: _buildConfirm(context)),
            ],
          ),
        )
      ],
    );
  }

  FWidget _buildTitle() {
    return FContainer(
      margin: EdgeInsets.symmetric(vertical: 15),
      color: Colors.white,
      constraints: BoxConstraints(maxWidth: 280),
      child: FColumn(
        children: [
          FText(
            title,
            style: TextStyle(fontSize: 16),
          ),
          FSizedBox(height: 8),
          if (subTitle != null)
            FText(
              subTitle!,
              style: TextStyle(fontSize: 14, color: Colors.black54),
            ),
        ],
      ),
    );
  }

  FWidget _buildCancel(BuildContext context) {
    final isPrimary = isCancelPrimary ?? false;
    var shape = RoundedRectangleBorder();
    return FTextButton(
      name: title,
      style: ButtonStyle(
        side: MaterialStateProperty.all(
          BorderSide.none,
        ),
        shape: MaterialStateProperty.all(
          shape.copyWith(
            borderRadius: BorderRadius.only(
              bottomLeft: Radius.circular(8.0),
            ),
          ),
        ),
        fixedSize: MaterialStateProperty.all(Size(double.infinity, 48)),
        backgroundColor: MaterialStateProperty.all(
            isPrimary ? FTheme.ins.colorScheme.primary : null),
      ),
      onPressed: () {
        Navigator.of(context).pop();
        onCancel?.call();
      },
      businessParent: businessParent,
      child: FText(
        cancelLabel ?? "取消",
        style: TextStyle(color: isPrimary ? Colors.white : Colors.grey),
      ),
    );
  }

  FWidget _buildConfirm(BuildContext context) {
    final isReadOnly = isConfirmReadonly ?? false;
    return FTextButton(
      style: TextButton.styleFrom(
        shape: RoundedRectangleBorder(),
        fixedSize: Size(double.infinity, 48),
      ),
      onPressed: isReadOnly
          ? null
          : () {
              Navigator.of(context).pop();
              onConfirm?.call();
              logger.i("$title $subTitle confirm");
            },
      name: '',
      businessParent: businessParent,
      child: FText(confrimLabel ?? "确认"),
    );
  }

  /// 显示确认弹框
  ///
  /// [title] 标题
  ///
  /// [subTitle] 副标题
  ///
  /// [confrimLabel] 确认文字
  ///
  /// [cancelLabel] 取消文字
  ///
  /// [onConfirm] 确认事件
  ///
  /// [onCancel] 取消事件
  static void show({
    required BuildContext context,
    required String title,
    String? subTitle,
    String? confrimLabel,
    String? cancelLabel,
    VoidCallback? onConfirm,
    VoidCallback? onCancel,
    bool? isConfirmReadonly,
    bool? isCancelPrimary,
    bool isShowCancel = true,
    required FInteractiveContainer businessParent,
  }) {
    showGeneralDialog(
      context: context,
      barrierDismissible: false,
      transitionBuilder: (context, a0, a1, child) {
        return Transform.scale(
          scale: a0.value,
          child: Opacity(
            opacity: a0.value,
            child: child,
          ),
        );
      },
      pageBuilder: (_, a0, a1) {
        return FConfirmAlert(
          title: title,
          subTitle: subTitle,
          isShowCancel: isShowCancel,
          confrimLabel: confrimLabel,
          cancelLabel: cancelLabel,
          onConfirm: () {
            if (onConfirm != null) {
              onConfirm.call();
              logger.i("$title $subTitle onConfirm");
            }
          },
          onCancel: onCancel,
          isCancelPrimary: isCancelPrimary,
          isConfirmReadonly: isConfirmReadonly,
          businessParent: businessParent,
        );
      },
    );
    return;
  }

  /// 显示确认弹框
  ///
  /// [title] 标题
  ///
  /// [subTitle] 副标题
  ///
  /// [confrimLabel] 确认文字
  ///
  /// [cancelLabel] 取消文字
  ///
  /// [onConfirm] 确认事件
  ///
  /// [onCancel] 取消事件
  static Future<void> showModalDialog({
    required BuildContext context,
    required String title,
    String? subTitle,
    String? confrimLabel,
    String? cancelLabel,
    VoidCallback? onConfirm,
    VoidCallback? onCancel,
    bool? isConfirmReadonly,
    bool? isCancelPrimary,
    required FInteractiveContainer businessParent,
  }) async {
    await showGeneralDialog(
      context: context,
      barrierDismissible: false,
      transitionBuilder: (context, a0, a1, child) {
        return Transform.scale(
          scale: a0.value,
          child: Opacity(
            opacity: a0.value,
            child: child,
          ),
        );
      },
      pageBuilder: (_, a0, a1) {
        return FConfirmAlert(
          title: title,
          subTitle: subTitle,
          confrimLabel: confrimLabel,
          cancelLabel: cancelLabel,
          onConfirm: () {
            if (onConfirm != null) {
              onConfirm.call();
              logger.i("$title $subTitle onConfirm");
            }
          },
          onCancel: onCancel,
          isCancelPrimary: isCancelPrimary,
          isConfirmReadonly: isConfirmReadonly,
          businessParent: businessParent,
        );
      },
    );
  }
}
