import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';

/// 分页组件
class FTablePagination extends StatefulWidget implements FWidget {
  const FTablePagination({
    super.key,
    this.currListLength,
    this.onChangePage,
    this.pageSize = 10,
    this.mainAxisAlignment = MainAxisAlignment.end,
    this.currPageIndex = 1,
    this.color = Colors.grey,
    this.totalItemNumText = '',
    this.previousFivePageText = '-5',
    this.nextFivePageText = '+5',
  });

  final Color color;

  /// 对齐方式
  final MainAxisAlignment mainAxisAlignment;

  /// 当前列表总数
  final int? currListLength;

  /// 切换当前页的方法
  final Function(int pageIndex, int listLength)? onChangePage;

  /// 当前页面显示的列表数
  final int? pageSize;

  /// 当前选中页数
  final int? currPageIndex;

  /// 页码总数文案
  final String totalItemNumText;

  /// 向前五页文案
  final String previousFivePageText;

  /// 向后五页文案
  final String nextFivePageText;

  @override
  State<FTablePagination> createState() => _TablePaginationState();
}

class _TablePaginationState extends State<FTablePagination> {
  late int currCheckedIndex = widget.currPageIndex!;

  /// 最大页数长度
  int maxPageLength = 0;
  bool get isFirstPage => currCheckedIndex == 1;
  bool get isLastPage =>
      currCheckedIndex == _buildPageButtonList(widget.currListLength ?? 0).last;
  @override
  void didUpdateWidget(covariant FTablePagination oldWidget) {
    super.didUpdateWidget(oldWidget);
    currCheckedIndex = widget.currPageIndex!;
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: widget.mainAxisAlignment,
      children: [
        _buildTotalContent(),
        _buildIconButton(
          Icon(
            Icons.chevron_left_rounded,
            color: isFirstPage ? Colors.grey : widget.color,
          ),
          isFirstPage,
          _previousPage,
        ),
        _buildPageButtonGroup(),
        _buildIconButton(
          Icon(
            Icons.chevron_right_rounded,
            color: isLastPage ? Colors.grey : widget.color,
          ),
          isLastPage,
          _nextPage,
        ),
        SizedBox(
          width: 20,
        )
      ],
    );
  }

  /// 上一页
  void _previousPage() {
    currCheckedIndex--;
    _onChangePage();
  }

  /// 下一页
  void _nextPage() {
    currCheckedIndex++;
    _onChangePage();
  }

  /// 切换当前页数
  void _onChangePage() {
    widget.onChangePage!(currCheckedIndex, widget.pageSize!);
    setState(() {});
  }

  /// 当前表格里面的总数
  Widget _buildTotalContent() {
    return Container(
      margin: const EdgeInsets.only(right: 4, top: 10),
      height: 30,
      alignment: Alignment.center,
      child: Text(
        widget.totalItemNumText,
        textAlign: TextAlign.end,
        style: TextStyle(color: widget.color),
      ),
    );
  }

  /// 当前页数的按钮 控制当前页面显示的页数或者显示...
  Widget _buildPageButtonGroup() {
    int currListLength = widget.currListLength ?? 0;
    List currList = _buildPageButtonList(currListLength);
    List<Widget> children = [];
    for (int i = 0; i < currList.length; i++) {
      if (currCheckedIndex == currList[i]) {
        children.add(_buildPageButton(
          Text(
            '${currList[i]}',
            style: const TextStyle(color: Color(0xff2c77e5)),
          ),
          currList[i],
        ));
      } else {
        if (currList[i] != '...') {
          children.add(_buildPageButton(
            Text(
              '${currList[i]}',
              style: TextStyle(color: widget.color),
            ),
            currList[i],
          ));
        } else {
          children.add(_buildEllipsisButton(FText('${currList[i]}'), i));
        }
      }
    }

    return Row(
      children: children,
    );
  }

  /// 图标按钮
  Widget _buildIconButton(
    Widget currPageWidge,
    bool isDisable,
    Function iconTap,
  ) {
    return Container(
      margin: const EdgeInsets.only(right: 4, top: 10),
      child: isDisable
          ? MouseRegion(
              cursor: SystemMouseCursors.forbidden,
              child: _buildIconButtonItem(
                currPageWidge,
                isDisable,
              ),
            )
          : InkWell(
              onTap: () {
                iconTap();
                setState(() {});
              },
              child: _buildIconButtonItem(
                currPageWidge,
                isDisable,
              ),
            ),
    );
  }

  /// 左右跳转按钮
  Widget _buildIconButtonItem(
    Widget currPageWidge,
    bool isDisable,
  ) {
    return Container(
      height: 30,
      constraints: BoxConstraints(
        minWidth: 30,
      ),
      alignment: Alignment.center,
      decoration: BoxDecoration(
        border: Border.all(
          color: isDisable ? Colors.grey : const Color(0xffd3d3d3),
        ),
      ),
      child: currPageWidge,
    );
  }

  /// 页面选择按钮
  Widget _buildPageButton(Widget currPageWidge, [int? currIndex]) {
    return Container(
      margin: const EdgeInsets.only(right: 4, top: 10),
      child: InkWell(
        onTap: () {
          if (currIndex != null) {
            currCheckedIndex = currIndex.toInt();
          }
          _onChangePage();
        },
        child: Container(
          height: 30,
          constraints: BoxConstraints(
            minWidth: 30,
          ),
          alignment: Alignment.center,
          decoration: BoxDecoration(
            border: Border.all(
              color: currCheckedIndex == currIndex
                  ? const Color(0xff2c77e5)
                  : const Color(0xffd3d3d3),
            ),
          ),
          child: currPageWidge,
        ),
      ),
    );
  }

  /// 省略号按钮
  Widget _buildEllipsisButton(FWidget currPageWidge, int currIndex) {
    String ellipsisTooltipText = '';
    if (currIndex < 4) {
      ellipsisTooltipText = widget.previousFivePageText;
    } else {
      ellipsisTooltipText = widget.nextFivePageText;
    }
    return Container(
      margin: const EdgeInsets.only(right: 4, top: 10),
      child: InkWell(
        onTap: () {
          if (currIndex < 4) {
            currCheckedIndex -= 5;
          } else {
            currCheckedIndex += 5;
          }
          if (currCheckedIndex > maxPageLength) {
            currCheckedIndex = maxPageLength;
          } else if (currCheckedIndex < 1) {
            currCheckedIndex = 1;
          }
          _onChangePage();
        },
        child: FTextTooltip(
          message: ellipsisTooltipText,
          margin: EdgeInsets.all(5),
          decoration: BoxDecoration(
            color: Colors.black.withOpacity(0.8),
            borderRadius: BorderRadius.circular(4),
          ),
          child: FContainer(
            width: 30,
            height: 30,
            alignment: Alignment.center,
            decoration: BoxDecoration(
              border: Border.all(
                color: const Color(0xffd3d3d3),
              ),
            ),
            child: currPageWidge,
          ),
        ),
      ),
    );
  }

  /// 当前页面列表的页数
  List _buildPageButtonList(int listLength) {
    List pageButtonList = [];
    int pageLength;
    if (listLength % widget.pageSize! == 0) {
      pageLength = listLength ~/ widget.pageSize!;
    } else {
      pageLength = (listLength / widget.pageSize!).ceil();
    }

    if (pageLength <= 1) {
      // 当页码数量小于等于1时就不需要页码了，直接跳过，此时list为空数组
      return [1];
    }
    if (pageLength <= 7) {
      // 7：是我自己规定一行内展示的页码item数量，需要展示多少可以自己定义，当小于等于7时，直接进行展示这7个页码item
      for (var i = 0; i < pageLength; i++) {
        pageButtonList.add(i + 1);
      }
    } else if (currCheckedIndex <= 4) {
      // 当页码大于7后，则需要根据当前页码对有些页码进行隐藏
      pageButtonList = [1, 2, 3, 4, 5, "...", pageLength];
    } else if (currCheckedIndex > pageLength - 3) {
      // 以此类推
      pageButtonList = [
        1,
        2,
        "...",
        pageLength - 3,
        pageLength - 2,
        pageLength - 1,
        pageLength
      ];
    } else {
      pageButtonList = [
        1,
        "...",
        currCheckedIndex - 1,
        currCheckedIndex,
        currCheckedIndex + 1,
        "...",
        pageLength
      ];
    }
    maxPageLength = pageLength;
    return pageButtonList;
  }
}
