import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_ui/define.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';

const EdgeInsets _defaultInsetPadding =
    EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0);

/// A simple material design dialog.
///
/// A simple dialog offers the user a choice between several options. A simple
/// dialog has an optional title that is displayed above the choices.
///
/// Choices are normally represented using [SimpleDialogOption] widgets. If
/// other widgets are used, see [contentPadding] for notes regarding the
/// conventions for obtaining the spacing expected by Material Design.
///
/// For dialogs that inform the user about a situation, consider using an
/// [AlertDialog].
///
/// Typically passed as the child widget to [showDialog], which displays the
/// dialog.
///
/// {@animation 350 622  }
///
/// {@tool snippet}
///
/// In this example, the user is asked to select between two options. These
/// options are represented as an enum. The [showDialog] method here returns
/// a [Future] that completes to a value of that enum. If the user cancels
/// the dialog (e.g. by hitting the back button on Android, or tapping on the
/// mask behind the dialog) then the future completes with the null value.
///
/// The return value in this example is used as the index for a switch statement.
/// One advantage of using an enum as the return value and then using that to
/// drive a switch statement is that the analyzer will flag any switch statement
/// that doesn't mention every value in the enum.
///
/// ```dart
/// Future<void> _askedToLead() async {
///   switch (await showDialog<Department>(
///     context: context,
///     builder: (BuildContext context) {
///       return FSimpleDialog(
///         title: const Text('Select assignment'),
///         children: <Widget>[
///           SimpleDialogOption(
///             onPressed: () { Navigator.pop(context, Department.treasury); },
///             child: const Text('Treasury department'),
///           ),
///           SimpleDialogOption(
///             onPressed: () { Navigator.pop(context, Department.state); },
///             child: const Text('State department'),
///           ),
///         ],
///       );
///     }
///   )) {
///     case Department.treasury:
///       // Let's go.
///       // ...
///     break;
///     case Department.state:
///       // ...
///     break;
///     case null:
///       // dialog dismissed
///     break;
///   }
/// }
/// ```
/// {@end-tool}
///
/// See also:
///
///  * [SimpleDialogOption], which are options used in this type of dialog.
///  * [AlertDialog], for dialogs that have a row of buttons below the body.
///  * [Dialog], on which [FSimpleDialog] and [AlertDialog] are based.
///  * [showDialog], which actually displays the dialog and returns its result.
///  * <https://material.io/design/components/dialogs.html#simple-dialog>
///
/// 参数 isUseColumnLayout 用于控制内容是否使用 Column 布局
class FSimpleDialog extends StatelessWidget implements FWidget {
  /// Creates a simple dialog.
  ///
  /// Typically used in conjunction with [showDialog].
  ///
  /// The [titlePadding] and [contentPadding] arguments must not be null.
  FSimpleDialog({
    Key? key,
    this.title,
    this.bottomButton,
    this.titlePadding = const EdgeInsets.fromLTRB(12.0, 12.0, 12.0, 12.0),
    this.titleTextStyle,
    this.children,
    this.contentPadding = const EdgeInsets.all(0.0),
    this.backgroundColor,
    this.elevation,
    this.semanticLabel,
    this.insetPadding = _defaultInsetPadding,
    this.clipBehavior = Clip.none,
    this.shape,
    this.isDefault = false,
    this.onOk,
    this.onCancel,
    this.okString,
    this.cancelString,
    this.onClose,
    this.alignment,
    this.isNeedClose = true,
    this.titleColor = const Color(0xff2c77e5),
    this.titleAxisAlignment = MainAxisAlignment.spaceBetween,
    this.isFConfirmAlert = false,
    this.isUseColumnLayout = false,
  }) : super(key: key);

  /// The (optional) title of the dialog is displayed in a large font at the top
  /// of the dialog.
  ///
  /// Typically a [Text] widget.
  final FWidget? title;

  ///标题颜色
  final Color titleColor;

  final FWidget? bottomButton;

  ///是否显示close图标
  final bool isNeedClose;

  //// 是否使用Column布局
  final bool isUseColumnLayout;

  /// Padding around the title.
  ///
  /// If there is no title, no padding will be provided.
  ///
  /// By default, this provides the recommend Material Design padding of 24
  /// pixels around the left, top, and right edges of the title.
  ///
  /// See [contentPadding] for the conventions regarding padding between the
  /// [title] and the [children].
  final EdgeInsetsGeometry titlePadding;

  /// Style for the text in the [title] of this [FSimpleDialog].
  ///
  /// If null, [DialogTheme.titleTextStyle] is used. If that's null, defaults to
  /// [TextTheme.headline6] of [ThemeData.textTheme].
  final TextStyle? titleTextStyle;

  /// The (optional) content of the dialog is displayed in a
  /// [SingleChildScrollView] underneath the title.
  ///
  /// Typically a list of [SimpleDialogOption]s.
  final List<Widget>? children;

  /// sure and
  final bool? isDefault;

  /// OK button event
  final VoidCallback? onOk;

  /// Cancel button event
  final VoidCallback? onCancel;

  /// Cancel button event
  final VoidCallback? onClose;

  /// OK  button string
  final String? okString;

  /// cancel  button string
  final String? cancelString;

  /// Padding around the content.
  ///
  /// By default, this is 12 pixels on the top and 16 pixels on the bottom. This
  /// is intended to be combined with children that have 24 pixels of padding on
  /// the left and right, and 8 pixels of padding on the top and bottom, so that
  /// the content ends up being indented 20 pixels from the title, 24 pixels
  /// from the bottom, and 24 pixels from the sides.
  ///
  /// The [SimpleDialogOption] widget uses such padding.
  ///
  /// If there is no [title], the [contentPadding] should be adjusted so that
  /// the top padding ends up being 24 pixels.
  final EdgeInsetsGeometry contentPadding;

  /// {@macro flutter.material.dialog.backgroundColor}
  final Color? backgroundColor;

  /// dialog size
  final Alignment? alignment;

  /// {@macro flutter.material.dialog.elevation}
  /// {@macro flutter.material.material.elevation}
  final double? elevation;

  /// The semantic label of the dialog used by accessibility frameworks to
  /// announce screen transitions when the dialog is opened and closed.
  ///
  /// If this label is not provided, a semantic label will be inferred from the
  /// [title] if it is not null.  If there is no title, the label will be taken
  /// from [MaterialLocalizations.dialogLabel].
  ///
  /// See also:
  ///
  ///  * [SemanticsConfiguration.namesRoute], for a description of how this
  ///    value is used.
  final String? semanticLabel;

  /// {@macro flutter.material.dialog.insetPadding}
  final EdgeInsets insetPadding;

  /// {@macro flutter.material.dialog.clipBehavior}
  final Clip clipBehavior;

  /// {@macro flutter.material.dialog.shape}
  final ShapeBorder? shape;

  /// 是否是确认提示框
  final bool? isFConfirmAlert;

  ///title的水平布局
  final MainAxisAlignment titleAxisAlignment;

  ScrollController _contentWidgetScrollController = ScrollController();

  static const C_STEPWIDTH = 56.0;
  static const C_BORDERRADIUS = 8.0;

  @override
  Widget build(BuildContext context) {
    assert(debugCheckHasMaterialLocalizations(context));
    final ThemeData theme = Theme.of(context);

    String? label = semanticLabel;
    if (title == null) {
      switch (theme.platform) {
        case TargetPlatform.macOS:
        case TargetPlatform.iOS:
          break;
        case TargetPlatform.android:
        case TargetPlatform.fuchsia:
        case TargetPlatform.linux:
        case TargetPlatform.windows:
          label =
              semanticLabel ?? MaterialLocalizations.of(context).dialogLabel;
      }
    }

    // The paddingScaleFactor is used to adjust the padding of Dialog
    // children.
    final double paddingScaleFactor =
        _paddingScaleFactor(MediaQuery.of(context).textScaleFactor);
    final TextDirection? textDirection = Directionality.maybeOf(context);

    Widget? titleWidget;
    if (title != null) {
      final EdgeInsets effectiveTitlePadding =
          titlePadding.resolve(textDirection);
      titleWidget = Padding(
        padding: EdgeInsets.only(),
        child: Row(
          mainAxisAlignment: titleAxisAlignment,
          children: [
            DefaultTextStyle(
              style: titleTextStyle ??
                  DialogTheme.of(context).titleTextStyle ??
                  theme.textTheme.headline6!,
              child: Semantics(
                namesRoute: label == null,
                container: true,
                child: title,
              ),
            ),
            if (isNeedClose) ...[
              IconButton(
                onPressed: () {
                  if (onClose != null) {
                    onClose?.call();
                    logger.i("$title dialog onCancel");
                  } else {
                    if (onCancel != null) {
                      onCancel?.call();
                    } else {
                      Navigator.of(context).pop();
                      logger.i("$title dialog back");
                    }
                  }
                },
                icon: Icon(
                  Icons.close,
                  size: 18,
                  color: Colors.white,
                ),
              ),
            ],
          ],
        ),
      );
    }

    Widget? contentWidget;
    if (children != null) {
      final EdgeInsets effectiveContentPadding =
          contentPadding.resolve(textDirection);
      if (isUseColumnLayout) {
        contentWidget = Expanded(
          child: Container(
            padding: EdgeInsets.only(
              left: effectiveContentPadding.left * paddingScaleFactor,
              right: effectiveContentPadding.right * paddingScaleFactor,
              top: title == null
                  ? effectiveContentPadding.top * paddingScaleFactor
                  : effectiveContentPadding.top,
              bottom: effectiveContentPadding.bottom * paddingScaleFactor,
            ),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: children!,
            ),
          ),
        );
      } else {
        contentWidget = Flexible(
          child: Scrollbar(
            controller: _contentWidgetScrollController,
            child: SingleChildScrollView(
              controller: _contentWidgetScrollController,
              padding: EdgeInsets.only(
                left: effectiveContentPadding.left * paddingScaleFactor,
                right: effectiveContentPadding.right * paddingScaleFactor,
                top: title == null
                    ? effectiveContentPadding.top * paddingScaleFactor
                    : effectiveContentPadding.top,
                bottom: effectiveContentPadding.bottom * paddingScaleFactor,
              ),
              child: ListBody(children: children!),
            ),
            isAlwaysShown: true,
          ),
        );
      }
    }

    Widget defaultBottom = Container(
      alignment: Alignment.center,
      padding: EdgeInsets.symmetric(vertical: 15),
      child: Flex(
        direction: Axis.horizontal,
        mainAxisSize: MainAxisSize.min,
        children: [
          ElevatedButton(
            onPressed: () {
              onOk?.call();
              logger.i("$title dialog onOk");
            },
            style: ButtonStyle(
              padding: MaterialStateProperty.all(
                _buttonPaddingEdgeInsets,
              ),
            ),
            child: Text(
              okString ?? '确定',
            ),
          ),
          SizedBox(
            width: 20,
          ),
          OutlinedButton(
            onPressed: () {
              onCancel?.call();
              logger.i("$title dialog onCancel");
            },
            style: ButtonStyle(
              backgroundColor: MaterialStateProperty.all(
                Colors.white,
              ),
              padding: MaterialStateProperty.all(
                _buttonPaddingEdgeInsets,
              ),
            ),
            child: Text(
              cancelString ?? '取消',
              style: TextStyle(
                color: Color.fromRGBO(61, 146, 252, 1),
              ),
            ),
          ),
        ],
      ),
    );

    Widget? bottomButtonsRow;
    if (bottomButton != null || isDefault!) {
      bottomButtonsRow = Container(
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            bottomLeft: Radius.circular(
              C_BORDERRADIUS,
            ),
            bottomRight: Radius.circular(
              C_BORDERRADIUS,
            ),
          ),
        ),
        child: isDefault! ? defaultBottom : bottomButton,
      );
    }

    Widget dialogChild = IntrinsicWidth(
      stepWidth: C_STEPWIDTH,
      child: ConstrainedBox(
        constraints: const BoxConstraints(minWidth: 280.0),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            if (title != null)
              Container(
                decoration: BoxDecoration(
                  color: titleColor,
                  borderRadius: BorderRadius.only(
                    topLeft: Radius.circular(C_BORDERRADIUS),
                    topRight: Radius.circular(C_BORDERRADIUS),
                  ),
                ),
                padding: isFConfirmAlert!
                    ? _titleFConfirmAlertPadding
                    : _titlePadding,
                child: titleWidget!,
              ),
            if (children != null) contentWidget!,
            if (bottomButton != null || isDefault!) ...[
              Container(
                decoration: BoxDecoration(
                  border: Border(
                    top: BorderSide(
                      color: Theme.of(context).dividerColor,
                      width: 0.5,
                    ),
                  ),
                ),
              ),
              Container(
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.only(
                    bottomLeft: Radius.circular(C_BORDERRADIUS),
                    bottomRight: Radius.circular(C_BORDERRADIUS),
                  ),
                ),
                child: bottomButtonsRow!,
              ),
            ]
          ],
        ),
      ),
    );

    if (label != null)
      dialogChild = Semantics(
        scopesRoute: true,
        explicitChildNodes: true,
        namesRoute: true,
        label: label,
        child: dialogChild,
      );
    return Dialog(
      backgroundColor: backgroundColor,
      elevation: elevation,
      alignment: alignment ?? Alignment.center,
      insetPadding: insetPadding,
      clipBehavior: clipBehavior,
      shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(C_BORDERRADIUS)),
      child: dialogChild,
    );
  }
}

EdgeInsets _titlePadding = EdgeInsets.only(
  left: 25,
  right: 6,
  top: 6,
  bottom: 6,
);
EdgeInsets _titleFConfirmAlertPadding = EdgeInsets.all(6);

EdgeInsets _buttonPaddingEdgeInsets = EdgeInsets.symmetric(
  vertical: 15,
  horizontal: 25,
);

double _paddingScaleFactor(double textScaleFactor) {
  final double clampedTextScaleFactor = textScaleFactor.clamp(1.0, 2.0);
  // The final padding scale factor is clamped between 1/3 and 1. For example,
  // a non-scaled padding of 24 will produce a padding between 24 and 8.
  return lerpDouble(1.0, 1.0 / 3.0, clampedTextScaleFactor - 1.0)!;
}
