import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';

/// ### 预排版按钮容器
/// - 用于将动态长度的按钮设置为宽度不变（取最宽的文案作为宽度）
///
/// 限制：
/// - 只能放入 Button 组件，当前支持的有：FElevatedButton、FOutlinedButton、FElevatedButtonWithIcon、FDelayedTiriggerButton
/// - 按钮的 child 当前只支持单个 FText 组件
/// - 按钮的 padding 需要设为0，如下：
/// ```dart
/// FOutlinedButton(
///   style: ButtonStyle(
///     padding: MaterialStatePropertyAll(EdgeInsets.all(0)),
///   ),
///   child: FText("Text..."),
///   ...
/// );
/// ```
/// e.g.
class FPrelayoutButtonWrapper extends FStatelessWidget {
  static const List<Type> supportedButtonTypes = [
    FElevatedButtonWithIcon,
    FElevatedButton,
    FOutlinedButton,
    FDelayedTiriggerButton,
  ];
  FPrelayoutButtonWrapper(
      {super.key,
      required this.child,
      required this.possibleValues,
      this.buttonPadding = 16.0})
      : assert(supportedButtonTypes.contains(child.runtimeType),
            'PrelayoutButtonWrapper children must be Supported'),
        assert(possibleValues.isNotEmpty, 'possibleTexts must not be empty');
  final FWidget child;

  /// 按钮水平内边距
  final double buttonPadding;

  /// 按钮组件自带的边框宽度
  static const double _defaultBorderWidth = 1.0;

  /// 可能出现的文案
  final List<String> possibleValues;

  /// 遍历获取所有可能出现的按钮宽度，然后取最大值
  double getMaxWidth() {
    double maxWidth = 0;
    for (var i = 0; i < possibleValues.length; i++) {
      String childTextValue = possibleValues[i];
      if (child is FElevatedButtonWithIcon) {
        final double innerSpacing = 8.0;
        var text = (child as FElevatedButtonWithIcon).label;
        var icon = (child as FElevatedButtonWithIcon).icon;
        var iconWidth = 24.0;
        if (icon is Icon) {
          iconWidth = getIconWidth(icon);
        }
        if (text is Text) {
          var width = getTextWidth(text, childTextValue) +
              iconWidth +
              innerSpacing +
              _defaultBorderWidth * 2;
          if (width > maxWidth) {
            /// 一般情况下 Icon 与 Text 之间存在间距，所以这里需要加上间距
            maxWidth = width;
          }
        }
      } else if (child is FElevatedButton) {
        var text = (child as FElevatedButton).child;
        if (text is Text) {
          var width =
              getTextWidth(text, childTextValue) + _defaultBorderWidth * 2;
          if (width > maxWidth) {
            maxWidth = width;
          }
        }
      } else if (child is FOutlinedButton) {
        var text = (child as FOutlinedButton).child;
        if (text is Text) {
          var width =
              getTextWidth(text, childTextValue) + _defaultBorderWidth * 2;
          if (width > maxWidth) {
            maxWidth = width;
          }
        }
      } else if (child is FDelayedTiriggerButton) {
        var text = (child as FDelayedTiriggerButton).child;
        if (text is Text) {
          var width =
              getTextWidth(text, childTextValue) + _defaultBorderWidth * 2 + 16;
          if (width > maxWidth) {
            maxWidth = width;
          }
        }
      }
    }
    return maxWidth;
  }

  double getTextWidth(Text textWidget, String textValue) {
    final TextStyle style = textWidget.style ?? TextStyle();
    var textPainter = TextPainter(
        text: TextSpan(
            style:
                style.copyWith(fontFamily: FTheme.ins.localeSetting.fontFamily),
            text: textValue),
        textDirection: TextDirection.ltr,
        maxLines: 1,
        textScaleFactor: 1);
    textPainter.layout();
    var width = textPainter.width;
    return width;
  }

  double getIconWidth(Icon icon) {
    final double width = icon.size ?? 24.0;
    return width;
  }

  double get totalWidth => getMaxWidth() + buttonPadding * 2;

  @override
  build(BuildContext context) {
    return FConstrainedBox(
      constraints: BoxConstraints.tightFor(width: totalWidth),
      child: child,
    );
  }
}
