import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';

import 'flutter_treeview/lib/flutter_treeview.dart';

const double _kDefaultLevelPadding = 20;
const int _kExpandSpeed = 130;

/// Defines the appearance of the [tree.TreeView].
///
/// Used by [tree.TreeView] to control the appearance of the sub-widgets
/// in the [tree.TreeView] widget.

class FTreeViewTheme extends TreeViewTheme {
  FTreeViewTheme({
    ColorScheme? colorScheme,
    double? levelPadding,
    bool? dense,
    double? verticalSpacing,
    double? horizontalSpacing,
    double? iconPadding,
    IconThemeData? iconTheme,
    ExpanderThemeData? expanderTheme,
    TextStyle? labelStyle,
    TextStyle? parentLabelStyle,
    TextOverflow? labelOverflow,
    TextOverflow? parentLabelOverflow,
    Duration? expandSpeed,
  }) : super(
          colorScheme: colorScheme ?? const ColorScheme.light(),
          iconTheme: iconTheme ?? const IconThemeData.fallback(),
          expanderTheme: expanderTheme ?? const ExpanderThemeData.fallback(),
          labelStyle: labelStyle ?? const TextStyle(),
          parentLabelStyle:
              parentLabelStyle ?? const TextStyle(fontWeight: FontWeight.bold),
          labelOverflow: labelOverflow,
          parentLabelOverflow: parentLabelOverflow,
          levelPadding: _kDefaultLevelPadding,
          dense: true,
          verticalSpacing: verticalSpacing,
          horizontalSpacing: horizontalSpacing,
          iconPadding: 8,
          expandSpeed: const Duration(milliseconds: _kExpandSpeed),
        );
}

class FTreeView extends StatelessWidget implements FWidget {
  /// The controller for the [tree.TreeView]. It manages the data and selected key.
  final TreeViewController controller;

  /// The tap handler for a node. Passes the node key.
  final Function(String)? onNodeTap;

  /// Custom builder for nodes. Parameters are the build context and tree node.
  final Widget Function(BuildContext, Node)? nodeBuilder;

  /// The double tap handler for a node. Passes the node key.
  final Function(String)? onNodeDoubleTap;

  /// The expand/collapse handler for a node. Passes the node key and the
  /// expansion state.
  final Function(String, bool)? onExpansionChanged;

  /// The theme for [tree.TreeView].
  final FTreeViewTheme? theme;

  /// Determines whether the user can select a parent node. If false,
  /// tapping the parent will expand or collapse the node. If true, the node
  /// will be selected and the use has to use the expander to expand or
  /// collapse the node.
  final bool allowParentSelect;

  /// How the [tree.TreeView] should respond to user input.
  final ScrollPhysics? physics;

  /// Whether the extent of the [tree.TreeView] should be determined by the contents
  /// being viewed.
  ///
  /// Defaults to false.
  final bool shrinkWrap;

  /// Whether the [tree.TreeView] is the primary scroll widget associated with the
  /// parent PrimaryScrollController..
  ///
  /// Defaults to true.
  final bool primary;

  /// Determines whether the parent node can receive a double tap. This is
  /// useful if [allowParentSelect] is true. This allows the user to double tap
  /// the parent node to expand or collapse the parent when [allowParentSelect]
  /// is true.
  /// ___IMPORTANT___
  /// _When true, the tap handler is delayed. This is because the double tap
  /// action requires a short delay to determine whether the user is attempting
  /// a single or double tap._
  final bool supportParentDoubleTap;
  FTreeView({
    Key? key,
    required this.controller,
    this.onNodeTap,
    this.onNodeDoubleTap,
    this.physics,
    this.onExpansionChanged,
    this.allowParentSelect: false,
    this.supportParentDoubleTap: false,
    this.shrinkWrap: false,
    this.primary: true,
    this.nodeBuilder,
    this.theme,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TreeView(
      controller: controller,
      onNodeTap: onNodeTap,
      onNodeDoubleTap: onNodeDoubleTap,
      onExpansionChanged: onExpansionChanged,
      allowParentSelect: allowParentSelect,
      supportParentDoubleTap: supportParentDoubleTap,
      shrinkWrap: shrinkWrap,
      primary: primary,
      nodeBuilder: nodeBuilder,
      theme: theme,
    );
  }
}
